/******************************************************************************
*       This module contains functions which handle interaction with the Voice 
*       Command and Voice Messages interface.
*
*       This code and information is provided "as is" without warranty
*       of any kind, either expressed or implied, including but not
*       limited to the implied warranties of merchantability and/or
*       fitness for a particular purpose.
*
Copyright (c) 1995-1997 by Microsoft Corporation

THE INFORMATION AND CODE PROVIDED HEREUNDER (COLLECTIVELY REFERRED TO
AS "SOFTWARE") IS PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. IN
NO EVENT SHALL MICROSOFT CORPORATION OR ITS SUPPLIERS BE LIABLE FOR
ANY DAMAGES WHATSOEVER INCLUDING DIRECT, INDIRECT, INCIDENTAL,
CONSEQUENTIAL, LOSS OF BUSINESS PROFITS OR SPECIAL DAMAGES, EVEN IF
MICROSOFT CORPORATION OR ITS SUPPLIERS HAVE BEEN ADVISED OF THE
POSSIBILITY OF SUCH DAMAGES. SOME STATES DO NOT ALLOW THE EXCLUSION OR
LIMITATION OF LIABILITY FOR CONSEQUENTIAL OR INCIDENTAL DAMAGES SO THE
FOREGOING LIMITATION MAY NOT APPLY.

This Software may be copied and distributed royalty-free subject to
the following conditions:

1. You must copy all Software without modification and must include
   all pages, if the Software is distributed without inclusion in your
   software product. If you are incorporating the Software in
   conjunction with and as a part of your software product which adds
   substantial value, you may modify and include portions of the
   Software.

2. You must place all copyright notices and other protective
   disclaimers and notices contained on the Software on all copies of
   the Software and your software product.

3. Unless the Software is incorporated in your software product which
   adds substantial value, you may not distribute this Software for
   profit.

4. You may not use Microsoft's name, logo, or trademarks to market
   your software product.

5. You agree to indemnify, hold harmless, and defend Microsoft and its
   suppliers from and against any claims or lawsuits, including
   attorneys' fees, that arise or result from the use or distribution
   of your software product and any modifications to the Software.
*/

#ifndef _SPEECH_
#define INITGUID
#define INC_OLE2
#endif  // _SPEECH_

#include "stdafx.h"
#include "shdocvw.h"
#include "webvox.h"
#include "webvoxdlg.h"
#include "vcmd.h"

BOOL ReleaseInterfaces( const char *szMsg );
HRESULT ActivateSR( int Flag );

HWND ghWndMain;

char *pszVoiceQuestion="Speech is turned off, you can turn this on using Feedback.Exe or Microsoft Voice.\nPress Yes after you've turned speech on, No to exit, or Cancel to continue.";

const int gnCmds = 10;	// the number of available commands for recognition
char *gszCmds=
"Quit Web Vox\nClose Web Vox\nOpen page\nGo Back\nGo Forward\nGo Home\nGo Search\nRefresh Page\nClose Browser\nOpen Browser";

PCIVCmdNotifySink gpVCmdNotifySink = NULL;
PCIVTxtNotifySink gpVTxtNotifySink = NULL;
PIVOICECMD        gpIVoiceCommand  = NULL;
PIVCMDDIALOGS     gpIVCmdDialogs   = NULL;
PIVCMDMENU        gpIVCmdMenu      = NULL;
PIVCMDATTRIBUTES  gpIVCmdAttr      = NULL;
PIVOICETEXT		  gpIVTxt          = NULL;
PIVTXTATTRIBUTES  gpIVTxtAttr      = NULL;
UINT              uTimerID = 0;


// registered messages for obtaining speech state changes...
UINT WM_REG_SPEECHSTARTED;
UINT WM_REG_SPEECHENDED;


char       *szWinTitle="WebVox";		// main window title
BOOL       bWidthFlag=FALSE;        // flag controlling window width/view
TEXTMETRIC TxtInfo;                 // text font info
HANDLE     hinst;                   // app instance
POINT      scrnsize;                // dimensions of the physical screen
RECT       rClient, rWin, rVMeter;  // client, window, and view meter bounding rectangles
POINT      TextPos;	                // positions of text information in wide view
SRTextOut  srTxt[100];      // wide view text data
PosSize_t  SRButPos, ClkButPos;     // clock/sr options button positions
int        BitsPerPixel;            // color resolution of current video mode
int        SRFlag = RECOG_GLOBAL;   // controls recognition active status
BOOL       bWndOnTop = FALSE;       // control for keeping window topmost
BOOL       bWndTitleBar = TRUE;     // window title on/off
BOOL       bVMeterFlag = TRUE;	    // view meter on/off
POINT      OrigSize, CurSize, BorderSize; // window size structures

/*
 Invoke the Speech Recognition general dialog.
*/
void ActivateSRdialog()
{
	HRESULT hRes = gpIVCmdDialogs->GeneralDlg( ghWndMain, "WebVox" );
}

/*
 Begin OLE and create the interface pointers and an empty menu object.

returns
   BOOL - TRUE if is succedes, FALSE if error.
*/
BOOL BeginOLE()
{
	HRESULT  hRes;
	VCMDNAME VcmdName;
	LANGUAGE Language;

	SetMessageQueue(96);

	if( FAILED(CoInitialize(NULL)) )
		return ReleaseInterfaces( "CoInitialize() failed." );

	// Create the voice commands object
	if(CoCreateInstance(CLSID_VCmd, NULL, CLSCTX_LOCAL_SERVER, IID_IVoiceCmd, (LPVOID *)&gpIVoiceCommand) != S_OK)
		return ReleaseInterfaces("Error in CoCreateInstance() for VCmd interface." );

	// Get the dialogs interface pointer...
	hRes = gpIVoiceCommand->QueryInterface( IID_IVCmdDialogs, (LPVOID FAR *)&gpIVCmdDialogs );
	if( FAILED(hRes) ) return ReleaseInterfaces( "QueryInterface() failed to obtain VCmd Dialogs interface." );

	// Get the attributes interface pointer...
	hRes = gpIVoiceCommand->QueryInterface( IID_IVCmdAttributes, (LPVOID FAR *)&gpIVCmdAttr );
	if( FAILED(hRes) ) return ReleaseInterfaces( "QueryInterface() failed to obtain VCmd Attributes interface." );

	// Create/Register VCmd notification sink...
	gpVCmdNotifySink = new CIVCmdNotifySink;
	if( gpVCmdNotifySink == NULL )
		return ReleaseInterfaces( "Out of memory for Voice notification object." );

	hRes = gpIVoiceCommand->Register( "", gpVCmdNotifySink, IID_IVCmdNotifySink, VCMDRF_ALLMESSAGES, NULL );
	if( FAILED(hRes) )
		return ReleaseInterfaces( "Failed to Register() Voice notification sink." );

	// Initialize command menu set variables...
	lstrcpy(VcmdName.szApplication, "WebVox");
	lstrcpy(VcmdName.szState, "Main");
	Language.LanguageID = LANG_ENGLISH;
	lstrcpy (Language.szDialect, "US English");

	// Create an empty command menu set...
	hRes = gpIVoiceCommand->MenuCreate( &VcmdName, &Language, VCMDMC_CREATE_TEMP, &gpIVCmdMenu );
	if( FAILED(hRes) ) return ReleaseInterfaces( "Failed to create a voice command set with MenuCreate()" );

	// Create a voice text object...
	if(CoCreateInstance(CLSID_VTxt, NULL, CLSCTX_LOCAL_SERVER, IID_IVoiceText, (LPVOID *)&gpIVTxt) != S_OK)
		return ReleaseInterfaces("Error in CoCreateInstance() for Text-To-Speech interface." );

	// Get the attributes interface pointer...
	hRes = gpIVTxt->QueryInterface( IID_IVTxtAttributes, (LPVOID FAR *)&gpIVTxtAttr );
	if( FAILED(hRes) ) return ReleaseInterfaces( "QueryInterface() failed to obtain VTxt Attributes interface." );

	// Create/Register VTxt notification sink...
	gpVTxtNotifySink = new CIVTxtNotifySink;
	if( gpVTxtNotifySink == NULL )
		return ReleaseInterfaces( "Out of memory for VTxt notification object." );

	hRes = gpIVTxt->Register( NULL, "WebVox", gpVTxtNotifySink, IID_IVTxtNotifySink, VTXTF_ALLMESSAGES, NULL);
	if( FAILED(hRes) )
		return ReleaseInterfaces( "Failed to Register() VTxt notification sink." );

	return TRUE;
}

/*
 Closes up OLE and frees everything else.
*/
BOOL EndOLE()
{
	
	ReleaseInterfaces( "" );

	// Free up all of OLE
	CoUninitialize();

	return TRUE;
}

/*
 Release the Voice command interfaces.
*/
BOOL ReleaseInterfaces( const char *szMsg )
{
	// if a msg is specified, display it...
	if( *szMsg ) MessageBox( NULL, szMsg, NULL, MB_OK);

	if( gpIVTxt         ) gpIVTxt->Release();
	if( gpIVCmdMenu     ) gpIVCmdMenu->Release();
	if( gpIVCmdAttr     ) gpIVCmdAttr->Release();
	if( gpIVCmdDialogs  ) gpIVCmdDialogs->Release();
	if( gpIVoiceCommand ) gpIVoiceCommand->Release();
	gpIVTxt          = NULL;
	gpIVCmdMenu      = NULL;
	gpIVCmdAttr      = NULL;
	gpIVCmdDialogs   = NULL;
	gpIVoiceCommand  = NULL;

	return FALSE;
}  

/*
 Scan memory and find the next command.

inputs
   CHAR     *pszMemory - Memory to start looking at
   PCHAR    *pBegin - Filled in with a pointer to the
         beginning of the command string.
   DWORD    *pdwSize - Filled in with the number of bytes in
         the string (excluding any NULL termination)
returns
   CHAR * - The next place that NextCommand should be called from,
         or NULL if no command string was found.
*/
CHAR * NextCommand (CHAR *pszMemory, PCHAR *pBegin, DWORD *pdwSize)
{
	DWORD i;

	for( ;; )
	{
		// try to find a non-newline
		while ((*pszMemory == '\n') || (*pszMemory == '\r'))
		{
			if( *pszMemory == '\0' ) return NULL;
			pszMemory++;
		}

		// Try to find a new-line
		for (i = 0; (pszMemory[i] != '\n') && (pszMemory[i] != '\r') && (pszMemory[i] != '\0'); i++);
		if( !i )
		{
			if( !pszMemory[i] ) return NULL;   // end
			pszMemory++;
			continue;   // try again
		}

		// Else, we've found a string
		*pBegin = pszMemory;
		*pdwSize = i;
		return pszMemory + i;
	}
}

/*
 Take a block of memory containing command strings and convert it into a list of VCMDCOMMAND
 structures.

inputs
   pszMemory       - NULL terminated string. Commands are separated by \n or \r.
   pData           - This is filled in with a pointer to memory and size for the vcmdcommand
                     structure. The memory must be freed by the caller with free().   
   *pdwNumCommands - Filled with the number of commands
*/
BOOL GetCommands(char *pszMemory, PSDATA pData, DWORD *pdwNumCommands)
{
	PSTR pTemp;
	DWORD dwTotal, dwSize, dwSizeDesc, dwSizeCat;
	DWORD dwSizeCmd;
	PVCMDCOMMAND pCmd, pCmdNew;
	CHAR    *pszBegin;
	DWORD   dwCmdSize;
	DWORD   dwCmds = 0;
	DWORD   dwCount = 1;
	char    szCat[] = "Main";

	dwTotal = dwSize = 0;

	pTemp = (PSTR)malloc(0);
	if( !pTemp ) return FALSE;

	pCmd = (PVCMDCOMMAND)pTemp;
	for( ;; )
	{
		pszMemory = NextCommand (pszMemory, &pszBegin, &dwCmdSize);
		if( !pszMemory ) break;   // no more

		// size of header
		dwSize = sizeof(VCMDCOMMAND);

		// get command length
		dwSizeCmd = (dwCmdSize + 1);

		// doubleword align
		dwSizeCmd += 3;
		dwSizeCmd &= (~3);
		dwSize += dwSizeCmd;

		// get description length
		dwSizeDesc = (dwCmdSize + 1);

		// doubleword align
		dwSizeDesc += 3;
		dwSizeDesc &= (~3);
		dwSize += dwSizeDesc;

		// get category length
		dwSizeCat = lstrlen(szCat) + 1;

		// doubleword align
		dwSizeCat += 3;
		dwSizeCat &= (~3);
		dwSize += dwSizeCat;

		// action indicator
		dwSize += sizeof(DWORD);

		// accumulate total size
		dwTotal += dwSize;

		// reallocate enough memory to hold this command
		pTemp = (PSTR)realloc((PVOID)pCmd, dwTotal);

		// fill in the new command
		pCmd = (PVCMDCOMMAND)pTemp;
		pTemp += (dwTotal-dwSize);
		pCmdNew = (PVCMDCOMMAND)pTemp;
		memset (pCmdNew, 0, dwSize);

		pCmdNew->dwSize = dwSize;
		pCmdNew->dwFlags = 0;
		pCmdNew->dwAction = (DWORD)(pCmdNew->abData-(PBYTE)pTemp);
		pCmdNew->dwActionSize = sizeof(DWORD);

		// point past header to begin of data
		pTemp += (pCmdNew->abData-(PBYTE)pTemp);

		// action index
		*(DWORD *)pTemp = dwCount++;
		pTemp += sizeof(DWORD);

		// command
		pCmdNew->dwCommand = (DWORD)((PBYTE)pTemp - (PBYTE)pCmdNew);
		strncpy(pTemp, pszBegin, dwCmdSize);
		pTemp += dwSizeCmd;

		// description
		pCmdNew->dwDescription = (DWORD)((PBYTE)pTemp - (PBYTE)pCmdNew);
		strncpy(pTemp, pszBegin, dwCmdSize);
		pTemp += dwSizeDesc;

		// category
		pCmdNew->dwCategory = (DWORD)((PBYTE)pTemp - (PBYTE)pCmdNew);
		strcpy(pTemp, szCat);

		// point to next pointers
		dwCmds++;
	}

	pData->pData = (PVOID)pCmd;
	pData->dwSize = dwTotal;
	*pdwNumCommands = dwCmds;
	return TRUE;
}

/*

inputs
   char     *pszState - State name
returns
   HRESULT - error
*/
HRESULT InitState ()
{
	HRESULT  hRes;
	SDATA    data;
	DWORD    dwNum, dwStart;
	DWORD    dwTotal=0;
	int      iState;

	// register our speech state messages...
	// this code is here to show you how to register these messages,
	// this app currently does not implement them completely...
	WM_REG_SPEECHSTARTED = RegisterWindowMessage( "WM_SPEECHSTARTED" );
	WM_REG_SPEECHENDED   = RegisterWindowMessage( "WM_SPEECHENDED" );
	if( WM_REG_SPEECHSTARTED == 0 || WM_REG_SPEECHENDED == 0 )
		MessageBox( NULL, "Can't register messages for obtaining the speech state.", "Warning", MB_OK );

	// add the recognizable cmds...
	if( !GetCommands(gszCmds, &data, &dwNum) )
	{
		MessageBox( NULL, "Out of memory during VCMD initialization.", NULL, MB_OK );
		return E_OUTOFMEMORY;
	}
	hRes = gpIVCmdMenu->Add (dwNum, data, &dwStart);
	free (data.pData);
	if( FAILED(hRes) )
	{
		MessageBox( NULL, "Error adding commands to menu.", NULL, MB_OK );
		return hRes;
	}


	// enable the VCMD and VTXT state and activate the menu,
	// note that this affects the entire site which means all voice-aware apps that
	// are running at this point...
	iState = 2;
	if( iState == 2 )
	{
		// VCMD and VCMD state...
		if( gpIVCmdAttr->EnabledSet(1) || gpIVCmdAttr->AwakeStateSet(1) )
			MessageBox( NULL, "Error enabling TTS during VCMD initialization.", NULL, MB_OK );
		
		// TTS...
		if( gpIVTxtAttr->EnabledSet(1) )
			MessageBox( NULL, "Error enabling TTS during VCMD initialization.", NULL, MB_OK );
	}

	// begin in global recognition state...
	if( ActivateSR( RECOG_GLOBAL ) )
		MessageBox( NULL, "Error activating the SR command menu.", NULL, MB_OK );

	return hRes;
}

/*
 Activate the recognition mode for off, recognize in window, or recognize globally.
*/
HRESULT ActivateSR( int Flag )
{
	HRESULT hRes;

	// turn it off...
	hRes = gpIVCmdMenu->Deactivate();

	// now turn it on if user wants to recognize no matter what window is active...
	if( Flag == RECOG_GLOBAL )
		hRes = gpIVCmdMenu->Activate( NULL, 0 );

	// or if the user just wants it to recognize while it's window is active...
	else if( Flag == RECOG_WINDOW )
		hRes = gpIVCmdMenu->Activate( ghWndMain, 0 );

	return hRes;
}

/*
 Send the string out to be spoken by the Voice Text interface.
*/
HRESULT SpeakString( char *szSpeak )
{
	HRESULT hRes;

	if( *szSpeak == NULL ) return 0;

	// temporarily disable SR to speak some text.  We'll enable it again in the
	// VTxt notification SpeakingDone() function. Note that if the wave driver
	// being used supports low-priority mode we don't have to do this (nor would
	// we need to implement the IVTxt notification interface)...
	gpIVCmdAttr->EnabledSet( FALSE );

	hRes = gpIVTxt->Speak( szSpeak, VTXTSP_NORMAL, NULL );

	// if we failed to start tts, then enable sr to be listening again...
	if( FAILED(hRes) ) gpIVCmdAttr->EnabledSet( TRUE );

	return hRes;
}

/**************************************************************************
 *  Voice Command notification objects
 **************************************************************************/

CIVCmdNotifySink::CIVCmdNotifySink (void)
{
    m_dwRefCnt = 0;
}

CIVCmdNotifySink::~CIVCmdNotifySink (void)
{
}

STDMETHODIMP CIVCmdNotifySink::QueryInterface (REFIID riid, LPVOID *ppv)
{
	*ppv = NULL;

	// always return our IUnkown for IID_IUnknown
	if (IsEqualIID (riid, IID_IUnknown) || IsEqualIID(riid,IID_IVCmdNotifySink)) {
		*ppv = (LPVOID) this;
		AddRef();
		return NOERROR;
	}

	// otherwise, cant find
	return E_NOINTERFACE;
}

STDMETHODIMP_ (ULONG) CIVCmdNotifySink::AddRef (void)
{
	return ++m_dwRefCnt;
}

STDMETHODIMP_(ULONG) CIVCmdNotifySink::Release (void)
{
	if( --m_dwRefCnt == 0 )
	{
		// reference count is 0, delete the object...
		delete this;
		return 0;
	}
	return m_dwRefCnt;
}

/*
 Handle recognition notification.

 Inputs:
 dwID         - Id of this instance of Voice command object.
 pName        - 
 dwFlags      -
 dwActionSize -
 pAction      - The index of the command in the command list (1 based).
 dwNumLists   -
 pszListValue - 
 pszCommand   - 
*/
STDMETHODIMP CIVCmdNotifySink::CommandRecognize(DWORD dwID, PVCMDNAME pName, DWORD dwFlags,
   DWORD dwActionSize, PVOID pAction, DWORD dwNumLists, PTSTR pszListValues, PTSTR pszCommand)
{
	// indicate recognition status to user by changing the window title back to normal...
	SetWindowText( ghWndMain, szWinTitle );

	// write the recognized command to the screen...
	strcpy( srTxt[SRTXT_CMD].szStrOut, pszCommand );

	switch( *(DWORD *)pAction )
	{
		// user said "Quit WebVox" (first cmd) Quit app
		case 1:
			((CWebVoxApp *)AfxGetApp())->QuitApp();
			break;

		// "Close WebVox"
		case 2:
			((CWebVoxDlg *)(((CWebVoxApp *)AfxGetApp())->m_pMainWnd))->SendMessage(WM_CLOSE);
			break;

		// "Open Page"
		case 3:
						
			((CWebVoxDlg *)(((CWebVoxApp *)AfxGetApp())->m_pMainWnd))->OpenPage();

			break;

				// "Go back"
		case 4:
						
			((CWebVoxApp *)AfxGetApp())->m_pIE->GoBack();

			break;

				// "Go Forward"
		case 5:
						
			((CWebVoxApp *)AfxGetApp())->m_pIE->GoForward();

			break;

				// "Go Home"
		case 6:
						
			((CWebVoxApp *)AfxGetApp())->m_pIE->GoHome();
			((CWebVoxApp *)AfxGetApp())->m_pIE->SetVisible(TRUE);

			break;
			
			// "Go Search"
		case 7:
						
			((CWebVoxApp *)AfxGetApp())->m_pIE->GoSearch();
			((CWebVoxApp *)AfxGetApp())->m_pIE->SetVisible(TRUE);

			break;
				// "Refresh Page"
		case 8:
						
			((CWebVoxApp *)AfxGetApp())->m_pIE->Refresh();

			break;
				// "Close Browser"
		case 9:
						
			((CWebVoxApp *)AfxGetApp())->m_pIE->Quit();

			break;
				// "Open Browser"
		case 10:
			/*
			IDispatch * FAR* ppDisp;			
			if (((CWebVoxApp *)AfxGetApp())->m_pIE->get_Application(ppDisp)==E_NOINTERFACE)
			{
				((CWebVoxApp *)AfxGetApp())->m_pIE = new IWebBrowserApp;
				((CWebVoxApp *)AfxGetApp())->m_pIE->CreateDispatch("InternetExplorer.Application");
			}
			*/
			break;
	}

	return NOERROR;
}


/*
 Handle notifications which are for some other command not in out list.
*/
STDMETHODIMP CIVCmdNotifySink::CommandOther(PVCMDNAME pName, PTSTR pszCommand)
{
	// put a '??' in the window title to tell the user we didn't understand the command...
	char buf[30];
	wsprintf( buf, "%s ?", szWinTitle );
	SetWindowText( ghWndMain, buf );

	strcpy( srTxt[SRTXT_CMD].szStrOut, "??" );
	SpeakString("Command not understood");

	return NOERROR;
}

/*
 Handle notifiaction sent when recognition begins.
*/
STDMETHODIMP CIVCmdNotifySink::CommandStart(void)
{
	return NOERROR;
}

/*
 Handle notification sent when the menu is activated/deactivated.
*/
STDMETHODIMP CIVCmdNotifySink::MenuActivate(PVCMDNAME pName, BOOL bActivate)
{
	return NOERROR;
}

/*
 Handle notification sent when the attribute is changed.
*/
STDMETHODIMP CIVCmdNotifySink::AttribChanged(DWORD dwAttribute)
{
	return NOERROR;
}

/*
 Handle notification sent when the recognizer detects the beginning of an utterance.
*/
STDMETHODIMP CIVCmdNotifySink::UtteranceBegin(void)
{
	// indicate the status to the user by changing the window title...
	SetWindowText( ghWndMain, "Utter..." );

	// fill string for widw view mode...
	strcpy( srTxt[SRTXT_UTTER].szStrOut, "Begin" );
	
	return NOERROR;
}

/*
 Handle notification sent when the recognizer detects the end of an utterance.
*/
STDMETHODIMP CIVCmdNotifySink::UtteranceEnd()
{
	// if we're not in wide mode, indicate the status to the user
	// by changing the window title...
	SetWindowText( ghWndMain, "Utter End" );


	strcpy( srTxt[SRTXT_UTTER].szStrOut, "End" );

	return NOERROR;
}

/*
 Handle notification sent to indicate amplitude of incoming audio data.
*/
STDMETHODIMP CIVCmdNotifySink::VUMeter(WORD wLevel)
{
/*
	// paint the view meter abse3d on data from notification...
	if( bVMeterFlag )
	{
		PaintVMeter( wLevel );
	}
*/
		
	

	if (wLevel > 500)
		RedrawWindow(AfxGetApp()->m_pMainWnd->m_hWnd, NULL, NULL, RDW_INVALIDATE);

	return NOERROR;
}

/*
 Handle notification sent when the recognizer detects interference
*/
STDMETHODIMP CIVCmdNotifySink::Interference(DWORD dwType)
{
	return NOERROR;
}

/**************************************************************************
 *  Voice Message notification object
 **************************************************************************/

/*
The only reason we need to implement this notification interface is because
we want to know when we're done speaking so we can enable speech recognition
mode.  This is because we probably have a half-duplex wave device which doesn't
allow SR and TTS simultaneously.  If we had a full-duplex wave device we wouldn't
need to do this.
*/
CIVTxtNotifySink::CIVTxtNotifySink ()
{
    m_dwRefCnt = 0;
}

CIVTxtNotifySink::~CIVTxtNotifySink (void)
{
}

STDMETHODIMP CIVTxtNotifySink::QueryInterface (REFIID riid, LPVOID *ppv)
{
	*ppv = NULL;

	// always return our IUnkown for IID_IUnknown
	if (IsEqualIID (riid, IID_IUnknown) || IsEqualIID(riid,IID_IVTxtNotifySink))
	{
		*ppv = (LPVOID) this;
		AddRef();
		return NOERROR;
	}

	// otherwise, cant find
	return E_NOINTERFACE;
}

STDMETHODIMP_ (ULONG) CIVTxtNotifySink::AddRef (void)
{
	return ++m_dwRefCnt;
}

STDMETHODIMP_(ULONG) CIVTxtNotifySink::Release (void)
{
	if( --m_dwRefCnt == 0 )
	{
		// reference count is 0, delete the object...
		delete this;
		return 0;
	}
    return m_dwRefCnt;
}

STDMETHODIMP CIVTxtNotifySink::AttribChanged(DWORD dwAttribute)
{
	return NOERROR;
}

STDMETHODIMP CIVTxtNotifySink::SpeakingStarted(void)
{
	return NOERROR;
}

STDMETHODIMP CIVTxtNotifySink::SpeakingDone(void)
{
	// Enable speech recognition again.  Note that if the wave driver being used
	// supports low-priority mode or full-duplex we wouldn't have to do this...
	gpIVCmdAttr->EnabledSet( TRUE );

	return NOERROR;
}

STDMETHODIMP CIVTxtNotifySink::Visual(WCHAR cIPAPhoneme, char cEnginePhoneme, DWORD dwFlags, PTTSMOUTH pTTSMouth)
{
	return NOERROR;
}

STDMETHODIMP CIVTxtNotifySink::Speak(PSTR pszMessage, PSTR pszApplication, DWORD dwTyep)
{
	return NOERROR;
}

